@@ -266,7 +266,11 @@ class MarketAnalyzer:
             kpis_df['label'] = kpis_df.index
 
             def get_algo_type(row):
-                type = signal_utility.unjsonize_list_run_results_list([row['label']])[0]['signal_type']
+                type = ''
+                try:
+                    type = signal_utility.unjsonize_list_run_results_list([row['label']])[0]['signal_type']
+                except Exception as err:
+                    print("Parsing signal type error: {0}".format(err))
                 return type
 
             kpis_df['label_type'] = kpis_df.apply(get_algo_type, axis=1)
