@@ -12,7 +12,7 @@ from napoleontoolbox.rebalancing import allocation
 
 
 class AbstractRunner(ABC):
-    def __init__(self, starting_date = None, running_date = None, drop_token=None, dropbox_backup = True, save_model=False, supervision_npy_file_suffix='_supervision.npy', macro_supervision_npy_file_suffix='_macro_supervision.npy', features_saving_suffix='_features.npy', features_names_saving_suffix='_features_names.npy', returns_pkl_file_suffix='_returns.pkl', local_root_directory='../data/', user = 'napoleon', n_start = 252, transaction_costs = None, crypto_lo = False):
+    def __init__(self, starting_date = None, running_date = None, drop_token=None, dropbox_backup = True, save_model=False, supervision_npy_file_suffix='_supervision.npy', macro_supervision_npy_file_suffix='_macro_supervision.npy', features_saving_suffix='_features.npy', features_names_saving_suffix='_features_names.npy', returns_pkl_file_suffix='_returns.pkl', local_root_directory='../data/', user = 'napoleon', n_start = 252, transaction_costs = None, crypto_ticker = None):
         super().__init__()
         self.supervision_npy_file_suffix = supervision_npy_file_suffix
         self.macro_supervision_npy_file_suffix = macro_supervision_npy_file_suffix
@@ -38,29 +38,29 @@ class AbstractRunner(ABC):
         self.hrp_allocation = True
         self.hrc_allocation = True
         self.mdp_allocation = True
-        self.transaction_costs = None
-        self.cypto_lo = crypto_lo
-        if self.cypto_lo:
-            self.returns_pkl_file_name =  'crypto_lo_' + self.starting_date.strftime('%d_%b_%Y') + '_' + self.running_date.strftime('%d_%b_%Y') + self.returns_pkl_file_suffix
+        self.transaction_costs = transaction_costs
+        self.crypto_ticker = crypto_ticker
+        if self.crypto_ticker is not None:
+            self.returns_pkl_file_name =  self.crypto_ticker + '_' + self.starting_date.strftime('%d_%b_%Y') + '_' + self.running_date.strftime('%d_%b_%Y') + self.returns_pkl_file_suffix
         else :
             self.returns_pkl_file_name = self.running_date.strftime('%d_%b_%Y') + self.returns_pkl_file_suffix
     @abstractmethod
-    def runTrial(self,saver, seed,  n, s, low_bound, up_bound):
+    def runTrial(self,saver, seed,  n, s, low_bound, up_bound, leverage):
         pass
 
 
 class SimpleEnsemblingRunner(AbstractRunner):
-    def runTrial(self, saver, seed,  n, s, low_bound, up_bound):
+    def runTrial(self, saver, seed,  n, s, low_bound, up_bound, leverage):
         try:
-            self.wrappedRunTrial(saver, seed, n, s, low_bound, up_bound)
+            self.wrappedRunTrial(saver, seed, n, s, low_bound, up_bound, leverage)
         except Exception as e:
             print('################# : To investigate and relaunch')
             print('################# : To investigate and relaunch')
             print('################# : To investigate and relaunch')
             print(e)
 
-    def wrappedRunTrial(self, saver, seed,  n, s, low_bound, up_bound):
-        meArg = (seed, n, s, low_bound, up_bound)
+    def wrappedRunTrial(self, saver, seed,  n, s, low_bound, up_bound, leverage):
+        meArg = (seed, n, s, low_bound, up_bound, leverage)
         meArgList = list(meArg)
         meArgList = [str(it) for it in meArgList]
         savingKey = ''.join(meArgList)
@@ -74,10 +74,11 @@ class SimpleEnsemblingRunner(AbstractRunner):
         print('Launching computation with parameters : '+savingKey)
 
         df = pd.read_pickle(self.local_root_directory + self.returns_pkl_file_name)
+        if 'Date' in df.columns:
+            print(df.columns)
+            df['Date'] = pd.to_datetime(df['Date'])
+            df = df.set_index('Date')
 
-        print(df.columns)
-        df['Date'] = pd.to_datetime(df['Date'])
-        df = df.set_index('Date')
         df = df.fillna(method='ffill')
         df = df[df.index >= self.starting_date]
         print('max before filtering ' + str(max(df.index)))
@@ -98,8 +99,15 @@ class SimpleEnsemblingRunner(AbstractRunner):
                     print(e)
                     return 0.
                 return fees
-            costs_parameters = np.fromiter(map(lambda x: get_fees_rate(x, self.transaction_costs), df_ret.columns),
-                                           dtype=np.float64)        ### getting the transaction costs
+
+            if isinstance(self.transaction_costs,dict):
+                costs_parameters = np.fromiter(map(lambda x: get_fees_rate(x, self.transaction_costs), df_ret.columns),
+                                               dtype=np.float64)
+            elif isinstance(self.transaction_costs,float):
+                costs_parameters = np.ones(len(df_ret.columns)) * self.transaction_costs
+            else:
+                print('no transaction costs given')
+                costs_parameters = np.zeros(len(df_ret.columns))
 
         check_maxperf_existence = saver.checkRunExistence('mp_' + savingKey)
         if self.maximize_perf and not check_maxperf_existence:
@@ -113,10 +121,13 @@ class SimpleEnsemblingRunner(AbstractRunner):
                 normalize=True,
                 low_bound=low_bound,
                 up_bound=up_bound,
-                filtering_threshold=0.9
+                filtering_threshold=0.9,
+                transaction_costs = costs_parameters
             )
 
             weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
+            weight_normalized = weight_normalized*leverage
+
             #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
             # # Compute portfolio perf
             transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
@@ -139,10 +150,13 @@ class SimpleEnsemblingRunner(AbstractRunner):
                 normalize=True,
                 low_bound=low_bound,
                 up_bound=up_bound,
-                filtering_threshold=0.9
+                filtering_threshold=0.9,
+                transaction_costs = costs_parameters
             )
             df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
             weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
+            weight_normalized = weight_normalized*leverage
+
             #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
             # # Compute portfolio perf
             transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
@@ -155,6 +169,8 @@ class SimpleEnsemblingRunner(AbstractRunner):
             print('equally weighted')
             df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
             weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
+            weight_normalized = weight_normalized*leverage
+
             #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
             # # Compute portfolio perf
             transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
@@ -176,10 +192,13 @@ class SimpleEnsemblingRunner(AbstractRunner):
                 ret=False,
                 low_bound=low_bound,
                 up_bound=up_bound,
-                filtering_threshold=0.9
+                filtering_threshold=0.9,
+                transaction_costs = costs_parameters
             )
             df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
             weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
+            weight_normalized = weight_normalized*leverage
+
             #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
             # # Compute portfolio perf
             transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
@@ -201,10 +220,13 @@ class SimpleEnsemblingRunner(AbstractRunner):
                 normalize=True,
                 low_bound=low_bound,
                 up_bound=up_bound,
-                filtering_threshold=0.9
+                filtering_threshold=0.9,
+                transaction_costs = costs_parameters
             )
             df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
             weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
+            weight_normalized = weight_normalized*leverage
+
             #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
             # # Compute portfolio perf
             transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
@@ -226,10 +248,13 @@ class SimpleEnsemblingRunner(AbstractRunner):
                 low_bound=low_bound,
                 up_bound=up_bound,
                 ret=False,
-                filtering_threshold=0.9
+                filtering_threshold=0.9,
+                transaction_costs = costs_parameters
             )
             df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
             weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
+            weight_normalized = weight_normalized*leverage
+
             #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
             # # Compute portfolio perf
             transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
@@ -253,10 +278,13 @@ class SimpleEnsemblingRunner(AbstractRunner):
                 metric='mse',
                 up_bound=up_bound,
                 low_bound=low_bound,
-                filtering_threshold=0.9
+                filtering_threshold=0.9,
+                transaction_costs = costs_parameters
             )
             df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
             weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
+            weight_normalized = weight_normalized*leverage
+
             #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
             # # Compute portfolio perf
             transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
@@ -276,9 +304,12 @@ class SimpleEnsemblingRunner(AbstractRunner):
                 s=s,
                 up_bound=up_bound,
                 low_bound=low_bound,
+                transaction_costs = costs_parameters
             )
             df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
             weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
+            weight_normalized = weight_normalized*leverage
+
             #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
             # # Compute portfolio perf
             transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
@@ -300,10 +331,13 @@ class SimpleEnsemblingRunner(AbstractRunner):
                 drift=True,
                 up_bound=up_bound,
                 low_bound=low_bound,
-                filtering_threshold=0.9
+                filtering_threshold=0.9,
+                transaction_costs = costs_parameters
             )
             df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
             weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
+            weight_normalized = weight_normalized*leverage
+
             #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
             # # Compute portfolio perf
             transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
