@@ -103,7 +103,15 @@ class AbstractRunner(ABC):
         self.signals_list.sort()
 
         self.dates_stub = starting_date.strftime('%d_%b_%Y') + '_' + running_date.strftime('%d_%b_%Y')
-        self.saving_return_path = self.local_root_directory+self.underlying+ '_' + self.dates_stub + self.freqly_return_pkl_filename_suffix
+
+        if self.frequence == 'daily':
+            if self.delay == 0:
+                self.saving_return_path = self.local_root_directory+self.underlying+ '_' + self.dates_stub + self.freqly_return_pkl_filename_suffix
+            else :
+                self.saving_return_path = f'{self.local_root_directory}{self.underlying}_{self.dates_stub}_{self.delay}{self.freqly_return_pkl_filename_suffix}'
+        else :
+            self.saving_return_path = self.local_root_directory+self.underlying+ '_' + self.dates_stub + self.freqly_return_pkl_filename_suffix
+
 
         self.recompute_all = recompute_all
         self.db_path_suffix = db_path_suffix
