@@ -12,6 +12,10 @@ import hashlib
 import pickle
 
 from pathlib import Path
+from napoleontoolbox.parallel_run import launcher_utility
+
+from sklearn.metrics import accuracy_score
+
 
 def unjsonize_mapping_dataframe(mapping_df):
     human_readable_dictionary = {}
@@ -245,3 +249,30 @@ class ParallelRunResultAnalyzer():
     def read_pickled_list_from_dropbox(self, list_pkl_file_name = None):
         raw_data = self.dbx.download_pkl(pkl_file_name=list_pkl_file_name)
         return raw_data
+
+
+    def get_accuracy_results(self, user=None, frequence=None, ssj=None, algo=None, starting_date=None, running_date=None, db_path_suffix=None, freqly_return_pkl_filename_suffix = None, jsonized_output = True):
+        db_path_name = launcher_utility.get_database_name(user,frequence,ssj,algo,starting_date,running_date,db_path_suffix)
+        dates_stub = starting_date.strftime('%d_%b_%Y') + '_' + running_date.strftime('%d_%b_%Y')
+        if freqly_return_pkl_filename_suffix is None:
+            freqly_return_pkl_filename_suffix = '_' + frequence + '_returns.pkl'
+        return_pkl_path = '../data_'+frequence+'/'+ssj+ '_' + dates_stub + freqly_return_pkl_filename_suffix
+        print(f'getting returns associated with {db_path_name}')
+        print(f'reading return from file {return_pkl_path}')
+        returns_df = pd.read_pickle(return_pkl_path)
+        returns_df['close_return'] = returns_df['close'].pct_change()
+        print('merging signals and results')
+        signals_df = self.analyzeAllRunSignalsResults(jsonized_output=jsonized_output)
+        results = {}
+        for me_sig in signals_df.columns:
+            print(f'computing accuracy for {me_sig}')
+            acc_df = pd.merge(signals_df[me_sig], returns_df['close_return'],how = 'inner', left_index=True, right_index=True)
+            me_acc = accuracy_score(acc_df['close_return']>0,acc_df[me_sig]>0)
+            results[me_sig] = [me_acc]
+        #print(f'dictionnary results {results}')
+        results_df = pd.DataFrame(results).T
+        results_df.columns = ['accuracy']
+        results_df = results_df.sort_values(by='accuracy', ascending=False)
+        return results_df
+
+
