@@ -16,6 +16,7 @@ import time
 from napoleontoolbox.bot import binance_bot
 from datetime import timedelta
 
+import datetime
 # CryptoCompare
 
 sleeping_time = 0.5
@@ -141,6 +142,59 @@ def binance_get_position_margin_history(api, p, start_time, end_time):
     return bot.get_futures_positions_margin_change_history(['BTCUSDT','ETHUSDT'], start_time, end_time)
 
 
+def request_day_data_paquet(url, me_ts, ssj):
+    r = requests.get(url.format(ssj, me_ts))
+    dataframe = None
+    try:
+        dataframe = pd.DataFrame(json.loads(r.text)['Data'])
+    except Exception as e:
+        print('no data')
+    return dataframe
+
+def cryptocompare_get(ssj ='USDT',ssj_against='USD', daily_crypto_starting_day=None, daily_crypto_ending_day=None, aggreg ='1d'):
+    assert aggreg == '1d'
+    dates_stub = daily_crypto_starting_day.strftime('%d_%b_%Y') + '_' + daily_crypto_ending_day.strftime('%d_%b_%Y')
+
+    year = datetime.datetime.now().year
+    month = datetime.datetime.now().month
+    day = datetime.datetime.now().day
+    hour = datetime.datetime.utcnow().hour
+    ts = datetime.datetime(year, month, day, tzinfo=datetime.timezone.utc).timestamp() + hour * 3600
+    ts1 = ts - 2001 * 3600 * 24
+    ts2 = ts1 - 2001 * 3600 * 24
+    ts3 = ts2 - 2001 * 3600 * 24
+    ts4 = ts3 - 2001 * 3600 * 24
+    ts5 = ts4 - 2001 * 3600 * 24
+    ts6 = ts5 - 2001 * 3600 * 24
+    ts7 = ts6 - 2001 * 3600 * 24
+    ts8 = ts7 - 2001 * 3600 * 24
+
+    print('Loading data')
+    day_url_request = 'https://min-api.cryptocompare.com/data/histoday?fsym={}&tsym=' + ssj_against + '&toTs={}&limit=2000'
+    dataframe = None
+    for me_timestamp in [ts8, ts7, ts6, ts5, ts4, ts3, ts2, ts1, ts]:
+        print('waiting')
+        df = request_day_data_paquet(day_url_request, me_timestamp, ssj)
+        if df is not None:
+            if dataframe is not None:
+                dataframe = dataframe.append(df, ignore_index=True)
+            else:
+                dataframe = df.copy()
+
+    dataframe['time'] = pd.to_datetime(dataframe['time'], unit='s')
+    dataframe = dataframe.sort_values(by=['time'])
+    dataframe = dataframe.rename(columns={"time": "date"}, errors="raise")
+    dataframe = dataframe.set_index(dataframe['date'])
+    dataframe = dataframe.drop(columns=['date'])
+    print('size fetched')
+    print(dataframe.shape)
+    dataframe = dataframe[dataframe.index >= daily_crypto_starting_day]
+    dataframe = dataframe[dataframe.index <= daily_crypto_ending_day]
+    print('size filtered after ' + str(daily_crypto_starting_day))
+    print(dataframe.shape)
+    return dataframe
+
+
 #https://binance-docs.github.io/apidocs/spot/en/#wallet-endpoints
 def binance_get(api, p, type_, start_time, end_time = None, symbol = None, aggreg = None):
   '''
@@ -487,7 +541,308 @@ def save_equity_daily_data(data_df = None, ticker=None, provider = 'napoleon', s
     data_df.to_pickle(local_root_directory + filename)
 
 
-def fetch_all_necessary_data_b40lux(public_key = None, private_key= None, start_time= None, end_time= None, me_month=None, save_to_dropbox = False, dropbox_token = None, local_root_directory = './', recompute_all=True, by_pass_month=False):
+def fetch_all_necessary_data_b40(public_key = None, private_key= None, start_time= None, end_time= None, me_month=None, save_to_dropbox = False, dropbox_token = None, local_root_directory ='./', recompute_all=True, by_pass_month=False):
+    print('inside')
+    dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=dropbox_token, dropbox_backup=True)
+
+    start_time_stub = start_time.strftime('%d_%b_%Y')
+    end_time_stub = end_time.strftime('%d_%b_%Y')
+    filename_stub = f'{public_key}_{start_time_stub}_{end_time_stub}_Binance'
+    #filename_stub = f'{inv_year_dictionary[year]}_{inv_month_dictionary[month]}_{inv_market_dictionary[market]}'
+
+    report_pkl_file_name = f'{filename_stub}_report_b40.pkl'
+
+    trade_pkl_file_name = f'{filename_stub}_trades_b40.pkl'
+
+    walletHist_pkl_file_name = f'{filename_stub}_walletHist_b40.pkl'
+
+    positions_pkl_file_name = f'{filename_stub}_positions_b40.pkl'
+
+    transfers_pkl_file_name = f'{filename_stub}_transfers_b40.pkl'
+
+    ohlc_bnb_min_binance_pkl_file_name = f'{filename_stub}_ohlc_bnb_min_binance.pkl'
+
+
+    ohlc_usdt_day_cc_pkl_file_name = f'{filename_stub}_ohlc_usdt_day_cc.pkl'
+
+
+    if recompute_all:
+
+        ohlc_usdt_day_cc = cryptocompare_get(ssj ='USDT', ssj_against='USD', daily_crypto_starting_day=start_time, daily_crypto_ending_day=end_time, aggreg ='1d')
+
+        fut_transfers = binance_get_futures_transfer_histo(public_key, private_key, start_time, end_time)
+
+        actualPositions = binance_get_position_risk(public_key,private_key,start_time, end_time)
+        history_balance = binance_get_future_balance_history(public_key,private_key,start_time, end_time)
+
+        trades = binance_get(public_key, private_key, 'trade_history', start_time, end_time=end_time, symbol ='ETHUSDT').drop_duplicates()
+        walletHist = binance_get(public_key, private_key, 'wallet_history', start_time, end_time= end_time, symbol ='ETHUSDT')
+
+        walletHist = walletHist[~walletHist['incomeType'].isin(['API_REBATE'])]
+
+
+        ohlc_bnb_min_binance = binance_get(public_key, private_key, 'spot_klines', start_time, end_time=end_time, symbol ='BNBUSDT', aggreg ='1m')
+
+
+        history_balance = pd.merge(history_balance, ohlc_bnb_min_binance[['ts','open']], how='left', left_on = 'date', right_on = 'ts')
+
+
+        def convert_bnb_to_usdt(row):
+            print(row)
+            if row['asset'] == 'BNB':
+                return float(row['walletBalance']) * float(row['open'])
+            elif row['asset'] == 'USDT':
+                return float(row['walletBalance'])
+            else :
+                return np.nan
+
+
+        history_balance['convertedWalletBalance'] = history_balance.apply(convert_bnb_to_usdt, axis=1)
+
+        converted_history_balance = history_balance[['ts','convertedWalletBalance']].groupby('ts').sum()
+
+        if save_to_dropbox :
+            full_path = local_root_directory + trade_pkl_file_name
+            trades.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=trade_pkl_file_name, overwrite=True)
+#            os.remove(full_path)
+
+            full_path = local_root_directory + walletHist_pkl_file_name
+            walletHist.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=walletHist_pkl_file_name, overwrite=True)
+#            os.remove(full_path)
+
+            full_path = local_root_directory + ohlc_bnb_min_binance_pkl_file_name
+            ohlc_bnb_min_binance.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=ohlc_bnb_min_binance_pkl_file_name, overwrite=True)
+#            os.remove(full_path)
+
+            full_path = local_root_directory + report_pkl_file_name
+            converted_history_balance.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=report_pkl_file_name, overwrite=True)
+#            os.remove(full_path)
+
+            full_path = local_root_directory + positions_pkl_file_name
+            actualPositions.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=positions_pkl_file_name, overwrite=True)
+#            os.remove(full_path)
+
+            full_path = local_root_directory + ohlc_usdt_day_cc_pkl_file_name
+            ohlc_usdt_day_cc.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=ohlc_usdt_day_cc_pkl_file_name, overwrite=True)
+            #            os.remove(full_path)
+
+            if fut_transfers is not None:
+                full_path = local_root_directory + transfers_pkl_file_name
+                fut_transfers.to_pickle(full_path)
+                print(f'uploading to dropbox {full_path}')
+                dbx.upload(fullname=full_path, folder='', subfolder='', name=transfers_pkl_file_name, overwrite=True)
+                #            os.remove(full_path)
+
+
+    else:
+        # trades = dbx.download_pkl(trade_pkl_file_name)
+        # walletHist = dbx.download_pkl(walletHist_pkl_file_name)
+        # ohlc_bnb_min_binance = dbx.download_pkl(ohlc_bnb_min_binance_pkl_file_name)
+        # converted_history_balance = dbx.download_pkl(report_pkl_file_name)
+        # actualPositions = dbx.download_pkl(positions_pkl_file_name)
+
+        ohlc_usdt_day_cc = pd.read_pickle(local_root_directory+ohlc_usdt_day_cc_pkl_file_name)
+        trades = pd.read_pickle(local_root_directory+trade_pkl_file_name)
+        walletHist = pd.read_pickle(local_root_directory+walletHist_pkl_file_name)
+        ohlc_bnb_min_binance = pd.read_pickle(local_root_directory+ohlc_bnb_min_binance_pkl_file_name)
+        converted_history_balance = pd.read_pickle(local_root_directory+report_pkl_file_name)
+        actualPositions = pd.read_pickle(local_root_directory+positions_pkl_file_name)
+#        fut_transfers = pd.read_pickle(local_root_directory + transfers_pkl_file_name)
+
+    # ohlc_bnb_binance (minutes)
+    ohlc_bnb_min_binance = ohlc_bnb_min_binance.set_index('ts')
+    assert isinstance(ohlc_bnb_min_binance.index, pd.DatetimeIndex)
+    ohlc_bnb_min_binance.columns = [x + '_bnb_bnc' for x in ohlc_bnb_min_binance.columns]
+    ohlc_bnb_min_binance.index = ohlc_bnb_min_binance.index.tz_localize(None)
+
+
+    ##################################
+    #################
+    ################# Positions
+    #################
+    ##################################
+    print(actualPositions.shape)
+    pos_columns = [
+        'symbol',
+        'positionAmt',
+        'markPrice'
+    ]
+    actualPositions = actualPositions[pos_columns].copy()
+    actualPositions = actualPositions.rename(columns={
+        'symbol':'asset',
+        'positionAmt':'amount',
+        'markPrice':'price'
+    })
+
+    ##################################
+    #################
+    ################# Trades
+    #################
+    ##################################
+
+    trades_df = trades.rename(columns={
+        'symbol': 'Security Code',
+        'commissionAsset': 'Currency',
+        'side': 'Oper',
+        'realizedPnl':'PnL',
+        'qty': 'Amount',
+        'price': 'Price',
+        'commission': 'Fees',
+        'ts': 'Date'
+    })
+
+    trades_df['Nature'] = 'CRY'
+
+
+
+    def convert_symbol(row):
+
+        if row['Security Code'] == 'BTCUSDT':
+            return 'BTC'
+        elif row['Security Code'] == 'ETHUSDT':
+            return 'ETH'
+        else:
+            return np.nan
+
+
+    trades_columns = ['Nature', 'Security Code', 'Currency', 'Oper', 'Date', 'Amount', 'Price', 'PnL', 'Fees']
+    trades_df = trades_df[trades_columns]
+    trades_df['Security Code'] = trades_df.apply(convert_symbol, axis=1)
+
+    futures_income_history_df = walletHist[
+        walletHist['incomeType'] == 'FUNDING_FEE'].copy()
+    futures_income_history_df['Nature'] = 'CRY'
+    futures_income_history_df = futures_income_history_df.rename(columns={
+        'symbol': 'Security Code',
+        'asset': 'Currency',
+        'incomeType': 'Oper',
+        'income': 'Fees',
+        'ts': 'Date'
+    })
+    futures_income_history_columns = ['Nature', 'Security Code', 'Currency', 'Oper', 'Date', 'Fees']
+    futures_income_history_df = futures_income_history_df[
+        futures_income_history_columns]
+
+    futures_income_history_df['Security Code'] = futures_income_history_df.apply(convert_symbol, axis=1)
+    #binance_df = pd.concat([trades_df, futures_income_history_df])
+    binance_df = trades_df.copy()
+    funding_df= futures_income_history_df.copy()
+    binance_df['Date'] = pd.to_datetime(binance_df['Date'])
+    binance_df = binance_df.sort_values(by=['Date'])
+
+    binance_df['Fees'] = -binance_df['Fees'].astype(float)
+
+
+    filteredWallet = converted_history_balance.copy()
+    converted_history_balance['Date'] = converted_history_balance.index
+    filteredWallet['Date'] = filteredWallet.index
+    filteredWallet['month'] = filteredWallet['Date'].dt.month
+    filteredWallet['insideMonth'] = filteredWallet['month'] == me_month
+    filteredWallet['shiftedInsideMonth'] = filteredWallet['insideMonth'].shift(-1)
+    filteredWallet['shiftedInsideMonth'] = filteredWallet['shiftedInsideMonth'].fillna(True)
+    filteredWallet = filteredWallet[filteredWallet['shiftedInsideMonth']].copy()
+    filteredWallet = filteredWallet.drop(columns=['month', 'insideMonth', 'shiftedInsideMonth'])
+
+    filteredWallet = filteredWallet.iloc[[0, -1]]
+
+    # if not by_pass_month:
+    #     minDate = min(filteredWallet['Date'])
+    #     maxDate = max(filteredWallet['Date'])
+    # else :
+    minDate = min(converted_history_balance.index)
+    maxDate = max(converted_history_balance.index)
+
+    filtered_binance_df = binance_df[binance_df['Date'] >= minDate].copy()
+    filtered_binance_df = filtered_binance_df[filtered_binance_df['Date'] <= maxDate].copy()
+    funding_df = funding_df[funding_df['Date'] <= maxDate].copy()
+
+    def filter_me_month(data_df, filtering_month):
+        data_df['month'] = data_df['Date'].dt.month
+        data_df['insideMonth'] = data_df['month'] == filtering_month
+        data_df = data_df[data_df['insideMonth']].copy()
+        data_df = data_df.drop(columns=['month', 'insideMonth'])
+        return data_df
+
+    if not by_pass_month:
+        filtered_binance_df=filter_me_month(filtered_binance_df.copy(),me_month)
+        funding_df=filter_me_month(funding_df.copy(),me_month)
+
+    filteredWallet = filteredWallet.rename(columns = {
+        'convertedWalletBalance': 'wallet_balance_usdt'
+    })
+
+    converted_history_balance = converted_history_balance.rename(columns = {
+        'convertedWalletBalance': 'wallet_balance_usdt'
+    })
+
+    valo_columns=[
+        'Date',
+        'wallet_balance_usdt',
+    ]
+    b40_cols = ['Nature', 'Security Code', 'Currency', 'Description', 'Comment', 'Oper', 'Date', 'Value', 'Settlement',
+                'Amount', 'Price', 'Int. Rate', 'Meth', 'Excl', 'Last Coupon', 'Interest', 'Depo', 'Bank', 'Pmt Cur',
+                'Source',	'Investor', 'Fees', 'Bank']
+
+
+    for me_col in b40_cols:
+        if me_col not in filtered_binance_df.columns:
+            filtered_binance_df[me_col] = np.nan
+
+    filtered_binance_df = filtered_binance_df[b40_cols].copy()
+
+
+    finalFilteredWallet = filteredWallet[valo_columns].copy()
+    final_converted_history_balance = converted_history_balance[valo_columns].copy()
+
+    ohlc_usdt_day_cc = ohlc_usdt_day_cc[['close']]
+    ohlc_usdt_day_cc = ohlc_usdt_day_cc.rename(columns = {
+        'close': 'close_usdt_usd'
+    })
+
+    finalFilteredWallet = pd.merge(finalFilteredWallet, ohlc_usdt_day_cc, how='left', left_index=True, right_index=True)
+    final_converted_history_balance = pd.merge(final_converted_history_balance, ohlc_usdt_day_cc, how='left', left_index=True, right_index=True)
+    finalFilteredWallet['Date'] = finalFilteredWallet['Date'].dt.strftime('%m/%d/%Y')
+    final_converted_history_balance['Date'] = final_converted_history_balance['Date'].dt.strftime('%m/%d/%Y')
+
+    filtered_binance_df['Date'] = filtered_binance_df['Date'].dt.strftime('%m/%d/%Y')
+
+    filtered_binance_df['Value'] = filtered_binance_df['Date']
+    filtered_binance_df['Settlement'] = filtered_binance_df['Date']
+
+    filtered_binance_df['Depo'] = 'Binance'
+    filtered_binance_df['Bank'] = 'Binance'
+
+    filtered_binance_df['Pmt Cur'] = 'USDT'
+    filtered_binance_df['Source'] = 'Manual'
+
+    funding_df['Date'] = funding_df['Date'].dt.strftime('%m/%d/%Y')
+
+    funding_df['Value'] = funding_df['Date']
+    funding_df['Settlement'] = funding_df['Date']
+
+    funding_df['Depo'] = 'Binance'
+    funding_df['Bank'] = 'Binance'
+
+    funding_df['Pmt Cur'] = 'USDT'
+    funding_df['Source'] = 'Manual'
+
+    return filtered_binance_df, finalFilteredWallet, final_converted_history_balance, actualPositions, funding_df
+
+
+
+def fetch_all_necessary_data_granular(public_key = None, private_key= None, start_time= None, end_time= None, me_month=None, save_to_dropbox = False, dropbox_token = None, local_root_directory ='./', recompute_all=True, by_pass_month=False):
     print('inside')
     dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=dropbox_token, dropbox_backup=True)
 
