@@ -29,6 +29,8 @@ warnings.simplefilter("ignore")
 import os
 import datetime as dt
 
+import datetime
+
 from datetime import timedelta
 from napoleontoolbox.file_saver import dropbox_file_saver
 
@@ -554,9 +556,428 @@ def preprocess_bitmex_trades(trades):
     #          'total_fees_usd']].groupby('dt_floor').sum()
 
 
+def request_day_data_paquet(url, me_ts, ssj):
+    r = requests.get(url.format(ssj, me_ts))
+    dataframe = None
+    try:
+        dataframe = pd.DataFrame(json.loads(r.text)['Data'])
+    except Exception as e:
+        print('no data')
+    return dataframe
+
+def cryptocompare_get(ssj ='USDT',ssj_against='USD', daily_crypto_starting_day=None, daily_crypto_ending_day=None, aggreg ='1d'):
+    assert aggreg == '1d'
+    dates_stub = daily_crypto_starting_day.strftime('%d_%b_%Y') + '_' + daily_crypto_ending_day.strftime('%d_%b_%Y')
+
+    year = datetime.datetime.now().year
+    month = datetime.datetime.now().month
+    day = datetime.datetime.now().day
+    hour = datetime.datetime.utcnow().hour
+    ts = datetime.datetime(year, month, day, tzinfo=datetime.timezone.utc).timestamp() + hour * 3600
+    ts1 = ts - 2001 * 3600 * 24
+    ts2 = ts1 - 2001 * 3600 * 24
+    ts3 = ts2 - 2001 * 3600 * 24
+    ts4 = ts3 - 2001 * 3600 * 24
+    ts5 = ts4 - 2001 * 3600 * 24
+    ts6 = ts5 - 2001 * 3600 * 24
+    ts7 = ts6 - 2001 * 3600 * 24
+    ts8 = ts7 - 2001 * 3600 * 24
+
+    print('Loading data')
+    day_url_request = 'https://min-api.cryptocompare.com/data/histoday?fsym={}&tsym=' + ssj_against + '&toTs={}&limit=2000'
+    dataframe = None
+    for me_timestamp in [ts8, ts7, ts6, ts5, ts4, ts3, ts2, ts1, ts]:
+        print('waiting')
+        df = request_day_data_paquet(day_url_request, me_timestamp, ssj)
+        if df is not None:
+            if dataframe is not None:
+                dataframe = dataframe.append(df, ignore_index=True)
+            else:
+                dataframe = df.copy()
+
+    dataframe['time'] = pd.to_datetime(dataframe['time'], unit='s')
+    dataframe = dataframe.sort_values(by=['time'])
+    dataframe = dataframe.rename(columns={"time": "date"}, errors="raise")
+    dataframe = dataframe.set_index(dataframe['date'])
+    dataframe = dataframe.drop(columns=['date'])
+    print('size fetched')
+    print(dataframe.shape)
+    dataframe = dataframe[dataframe.index >= daily_crypto_starting_day]
+    dataframe = dataframe[dataframe.index <= daily_crypto_ending_day]
+    print('size filtered after ' + str(daily_crypto_starting_day))
+    print(dataframe.shape)
+    return dataframe
+
+def fetch_all_necessary_data_b40(public_key = None, private_key= None, start_time= None, end_time= None, me_month=None, save_to_dropbox = False, dropbox_token = None, local_root_directory ='./', recompute_all = True, by_pass_month = False):
+
+    dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=dropbox_token, dropbox_backup=True)
+
+    start_time_stub = start_time.strftime('%d_%b_%Y')
+    end_time_stub = end_time.strftime('%d_%b_%Y')
+    filename_stub = f'{public_key}_{start_time_stub}_{end_time_stub}_Bitmex'
+    #filename_stub = f'{inv_year_dictionary[year]}_{inv_month_dictionary[month]}_{inv_market_dictionary[market]}'
 
 
-def fetch_all_necessary_data_b40(public_key = None, private_key= None, start_time= None, end_time= None, me_month=None, save_to_dropbox = False, dropbox_token = None, local_root_directory = './', recompute_all = True, by_pass_month = False):
+    reportvalo_pkl_file_name = f'{filename_stub}_report_valo_b40.pkl'
+    trade_pkl_file_name = f'{filename_stub}_trades_b40.pkl'
+    processed_trade_pkl_file_name = f'{filename_stub}_processed_trades_b40.pkl'
+
+    positions_pkl_file_name = f'{filename_stub}_positions_b40.pkl'
+
+    ohlc_btc_cc_pkl_file_name = f'{filename_stub}_ohlc_btc_cc.pkl'
+
+    ohlc_usdt_day_cc_pkl_file_name = f'{filename_stub}_ohlc_usdt_day_cc.pkl'
+
+    if recompute_all:
+        ohlc_usdt_day_cc = cryptocompare_get(ssj ='USDT', ssj_against='USD', daily_crypto_starting_day=start_time, daily_crypto_ending_day=end_time, aggreg ='1d')
+
+        client = bitmex.bitmex(test=False, config=None, api_key=public_key, api_secret=private_key)
+
+        #getting all the wallet information
+        #wallet_valo_histo = pd.DataFrame(client.User.User_getWalletHistory().result()[0])
+        actual_positions = pd.DataFrame(client.Position.Position_get().result()[0])
+
+        if isinstance(start_time,dt.datetime) and isinstance(end_time,dt.datetime):
+            start_time = start_time.strftime('%Y-%m-%dT%H:%M')
+            end_time = end_time.strftime('%Y-%m-%dT%H:%M')
+
+        ce, te = parser.parse(start_time).replace(tzinfo=pytz.UTC), parser.parse(end_time).replace(tzinfo=pytz.UTC)
+
+        ce = ce - dt.timedelta(hours=8)
+        trades = pd.DataFrame()
+        print('Accessing Trade History...')
+        while ce < te:
+            print(f'requesting trades between {ce} and {te}')
+            tmp = pd.DataFrame(client.Execution.Execution_getTradeHistory(startTime=ce, count=500).result()[0]).sort_values(
+                by='timestamp', ascending=True)
+            trades = trades.append(tmp)
+            time.sleep(4)
+            ce = tmp.timestamp.iloc[-1]
+        trades = trades[trades.timestamp <= te].copy()
+        trades.execComm = - trades.execComm  # logique comptable
+        trades = trades.sort_values(by='timestamp', ascending=True).drop_duplicates()
+        print('Done trades.')
+
+        # Get wallet history for a period of time
+        ce, te = parser.parse(start_time).replace(tzinfo=pytz.UTC), parser.parse(end_time).replace(tzinfo=pytz.UTC)
+        ce = ce - dt.timedelta(hours=8)
+        walletHist = pd.DataFrame()
+        print('Accessing wallet history...')
+        while ce < te:
+            print(f'requesting wallet history between {ce} and {te}')
+            tmp = pd.DataFrame(client.User.User_getWalletHistory(currency='XBt').result()[0]).sort_values(by='timestamp',
+                                                                                                          ascending=True)
+            walletHist = walletHist.append(tmp)
+            time.sleep(3.5)
+            ce = tmp.timestamp.iloc[-1]
+        walletHist = walletHist[walletHist.timestamp <= te].copy()
+        print('Wallet History retrieved.')
+
+        # Récupérer les OHLC hourly sur Cryptocompare depuis startdate
+        cc = CryptoCompare('')
+        ohlc_btc_cc = cc.get('OHLC', 'BTC', start_time, end_time, precision='hour')
+
+        if save_to_dropbox :
+            full_path = local_root_directory + trade_pkl_file_name
+            trades.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=trade_pkl_file_name, overwrite=True)
+            #os.remove(full_path)
+
+            full_path = local_root_directory + reportvalo_pkl_file_name
+            walletHist.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=reportvalo_pkl_file_name, overwrite=True)
+            #os.remove(full_path)
+
+            full_path = local_root_directory + ohlc_btc_cc_pkl_file_name
+            ohlc_btc_cc.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=ohlc_btc_cc_pkl_file_name, overwrite=True)
+            #os.remove(full_path)
+
+            full_path = local_root_directory + positions_pkl_file_name
+            actual_positions.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=positions_pkl_file_name, overwrite=True)
+            #os.remove(full_path)
+
+            full_path = local_root_directory + ohlc_usdt_day_cc_pkl_file_name
+            ohlc_usdt_day_cc.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=ohlc_usdt_day_cc_pkl_file_name, overwrite=True)
+            #            os.remove(full_path)
+    else:
+        # trades=dbx.download_pkl(trade_pkl_file_name)
+        # walletHist=dbx.download_pkl(reportvalo_pkl_file_name)
+        # ohlc_btc_cc=dbx.download_pkl(ohlc_btc_cc_pkl_file_name)
+        # actual_positions=dbx.download_pkl(positions_pkl_file_name)
+
+
+        ohlc_usdt_day_cc = pd.read_pickle(local_root_directory+ohlc_usdt_day_cc_pkl_file_name)
+        trades=pd.read_pickle(local_root_directory +trade_pkl_file_name)
+        walletHist=pd.read_pickle(local_root_directory +reportvalo_pkl_file_name)
+        ohlc_btc_cc=pd.read_pickle(local_root_directory +ohlc_btc_cc_pkl_file_name)
+        actual_positions=pd.read_pickle(local_root_directory +positions_pkl_file_name)
+    ##############################
+    ##############################
+    ##########
+    ########## computing positions
+    ##########
+    ##############################
+    ##############################
+    print('usdt change rate fetched')
+    print(ohlc_usdt_day_cc.shape)
+
+    print(actual_positions.shape)
+    actual_positions.timestamp = actual_positions.timestamp.apply(lambda x: x.timestamp() * 1000)
+    actual_positions['timestamp'] = pd.to_datetime(actual_positions['timestamp'],unit='ms')
+    actual_positions['totalAsset']=actual_positions['currentQty']/actual_positions['lastPrice']
+    posColumns=[
+        'timestamp',
+        'symbol',
+        'lastPrice',
+        'totalAsset'
+    ]
+    actual_positions = actual_positions[posColumns].copy()
+    actualPositions = actual_positions.rename(columns={
+        'symbol':'asset',
+        'totalAsset':'amount',
+        'lastPrice':'price'
+    })
+
+    print('done')
+
+    ##############################
+    ##############################
+    ##########
+    ########## computing trades
+    ##########
+    ##############################
+    ##############################
+    reprocess_numba_trades = False
+    if reprocess_numba_trades:
+        trades = trades.sort_values(by='timestamp', ascending=True).drop_duplicates()
+        # trades.transactTime = trades.transactTime.apply(lambda x: x.timestamp() * 1000)
+        # trades['transactTime'] = pd.to_datetime(trades['transactTime'])
+        # trades = trades.drop(columns=['timestamp'])
+        # trades.to_excel('/Users/stefanduprey/Documents/My_SlippageFiles/b40lux_september_bitmex_raw_trades.xls',
+        #                 engine='xlsxwriter', index=False)
+        # raise Exception('mouteur foureur')
+        print('preprocessing trades')
+        bitmex_df = preprocess_bitmex_trades(trades)
+        print('preporcessing done')
+        print(bitmex_df.shape)
+        if save_to_dropbox:
+            full_path = local_root_directory + processed_trade_pkl_file_name
+            bitmex_df.to_pickle(full_path)
+            print(f'uploading to dropbox {full_path}')
+            dbx.upload(fullname=full_path, folder='', subfolder='', name=processed_trade_pkl_file_name, overwrite=True)
+            #            os.remove(full_path)
+    else:
+        bitmex_df=pd.read_pickle(local_root_directory +processed_trade_pkl_file_name)
+
+
+    ##############################
+    ##############################
+    ##########
+    ########## computing valos
+    ##########
+    ##############################
+    ##############################
+    # ohlc_btc_cc
+    x = ohlc_btc_cc.copy()
+    x.index = x.index.map(parser.parse)
+    x.index = x.index.tz_localize(None)
+    x.columns = [y + '_btc_cc' for y in x.columns]
+    ohlc_btc_cc = x.copy()
+
+    walletHist = walletHist.set_index('timestamp')
+    walletHist.index = walletHist.index.map(lambda x: x.floor('60s'))
+    walletHist = walletHist[~walletHist.index.duplicated(keep='first')].sort_index(ascending=True)
+    walletHist.index = walletHist.index.tz_localize(None)
+
+
+    walletHist['walletBalance_xbt'] = walletHist.walletBalance / 100000000
+    walletHist = walletHist.join(ohlc_btc_cc['open_btc_cc'])
+    walletHist['open_btc_cc'] = walletHist['open_btc_cc'].bfill()
+    walletHist['walletBalance_usd'] = walletHist.walletBalance_xbt * walletHist.open_btc_cc
+
+    def compute_signal_effective_date(row):
+        # if np.isnan(row['date']):
+        if pd.isnull(row['date']):
+            return np.nan
+        else:
+            return (datetime.date(row['date'].year, row['date'].month, row['date'].day)) #+timedelta(days = 1))
+
+    walletHist['date'] = pd.to_datetime(walletHist.index)
+    walletHist['date'] = walletHist.apply(compute_signal_effective_date, axis=1)
+    walletHist.index = walletHist['date']
+    walletHist = pd.merge(walletHist, ohlc_usdt_day_cc, left_index = True, right_index = True)
+
+
+    ########################################################################
+    ########################################################################
+    ################### gettingn the proper trades df
+
+    bitmex_df = bitmex_df.rename(columns={
+        'execType' : 'Nature',
+        'symbol': 'Security Code',
+        'side': 'Oper',
+        'quantity': 'Amount',
+        'price': 'Price',
+        'total_fees_usd': 'Fees',
+        'transactTime': 'Date'
+    })
+
+    funding_df = bitmex_df[bitmex_df['Nature'] == 'Funding'].copy()
+    bitmex_df = bitmex_df[bitmex_df['Nature'] == 'Trade'].copy()
+
+    bitmex_df['Nature'] = 'CRY'
+    bitmex_df['Currency'] = 'USDT'
+
+    funding_df['Nature'] = 'CRY'
+    funding_df['Currency'] = 'USDT'
+
+    trades_columns = ['Nature', 'Security Code', 'Currency', 'Oper', 'Date', 'Amount','Price', 'Fees']
+
+    bitmex_df = bitmex_df[trades_columns].copy()
+    funding_df = funding_df[trades_columns].copy()
+
+
+    def convert_symbol(row):
+        if row['Security Code'] == 'XBTUSD':
+            return 'BTC'
+        elif row['Security Code'] == 'ETHUSD':
+            return 'ETH'
+        else:
+            return np.nan
+
+    bitmex_df['Security Code'] = bitmex_df.apply(convert_symbol, axis=1)
+
+
+
+    if not by_pass_month:
+        bitmex_df['month'] = bitmex_df['Date'].dt.month
+        bitmex_df = bitmex_df[bitmex_df['month'] == me_month].copy()
+        bitmex_df = bitmex_df.drop(columns=['month'])
+
+    def filter_me_month(data_df, filtering_month):
+        data_df['month'] = data_df['Date'].dt.month
+        data_df['insideMonth'] = data_df['month'] == filtering_month
+        data_df = data_df[data_df['insideMonth']].copy()
+        data_df = data_df.drop(columns=['month', 'insideMonth'])
+        return data_df
+
+    if not by_pass_month:
+        filtered_bitmex_df=filter_me_month(bitmex_df.copy(),me_month)
+        funding_df=filter_me_month(funding_df.copy(),me_month)
+
+    ############## getting the proper wallet Hist
+    filteredWallet = walletHist.copy()
+    filteredWallet['Date'] = filteredWallet.index
+    filteredWallet['month'] = filteredWallet['Date'].dt.month
+    filteredWallet['insideMonth'] = filteredWallet['month'] == me_month
+    filteredWallet['shiftedInsideMonth'] = filteredWallet['insideMonth'].shift(-1)
+    filteredWallet['shiftedInsideMonth'] = filteredWallet['shiftedInsideMonth'].fillna(True)
+    filteredWallet = filteredWallet[filteredWallet['shiftedInsideMonth']].copy()
+    filteredWallet = filteredWallet.drop(columns=['month', 'insideMonth', 'shiftedInsideMonth'])
+
+    filteredWallet = filteredWallet.iloc[[0, -1]]
+
+    if by_pass_month:
+        minDate = min(walletHist.index)
+        maxDate = max(walletHist.index)
+
+        filtered_bitmex_df = bitmex_df[bitmex_df['Date'] >= minDate].copy()
+        filtered_bitmex_df = filtered_bitmex_df[filtered_bitmex_df['Date'] <= maxDate].copy()
+
+        walletHist_df = walletHist[walletHist.index >= minDate].copy()
+        walletHist_df = walletHist_df[walletHist_df.index <= maxDate].copy()
+    else :
+        walletHist_df = walletHist.copy()
+    walletHist_df['Date'] = walletHist_df.index
+
+    walletHist_df = walletHist_df.sort_values(by = 'Date')
+    #######
+
+
+    filtered_bitmex_df = filtered_bitmex_df.sort_values(by = 'Date')
+    valo_columns=[
+        'open_btc_cc',
+        'close',
+        'walletBalance_xbt',
+        'walletBalance_usd',
+        'Date'
+    ]
+    #assert(filtered_bitmex_df['Value'].sum() == walletHist_df['realizedPnl'].sum())
+    walletHist_df['Date'] = walletHist_df.index
+
+    filteredWallet = filteredWallet[valo_columns].copy()
+
+    filteredWallet = filteredWallet.rename(columns = {
+        'walletBalance_xbt':'wallet_balance_btc',
+        'walletBalance_usd': 'wallet_balance_usdt',
+        'open_btc_cc':'btc_price',
+        'close':'rate_usdt_usd'
+    })
+
+    walletHist_df = walletHist_df.rename(columns = {
+        'walletBalance_xbt':'wallet_balance_btc',
+        'walletBalance_usd': 'wallet_balance_usdt',
+        'open_btc_cc':'btc_price',
+        'close':'rate_usdt_usd'
+    })
+
+    valo_columns=[
+        'Date',
+        'btc_price',
+        'wallet_balance_usdt',
+        'wallet_balance_btc',
+        'rate_usdt_usd'
+    ]
+
+    b40_cols = ['Nature', 'Security Code', 'Currency', 'Description', 'Comment', 'Oper', 'Date', 'Value', 'Settlement',
+                'Amount', 'Price', 'Int. Rate', 'Meth', 'Excl', 'Last Coupon', 'Interest', 'Depo', 'Bank', 'Pmt Cur',
+                'Source	Investor', 'Fees', 'Bank', 'PnL']
+    for me_col in b40_cols:
+        if me_col not in filtered_bitmex_df.columns:
+            filtered_bitmex_df[me_col] = np.nan
+
+    filtered_bitmex_df = filtered_bitmex_df[b40_cols].copy()
+    filWal = filteredWallet[valo_columns].copy()
+    wal =  walletHist_df[valo_columns]
+
+    filtered_bitmex_df = filtered_bitmex_df.sort_values(by=['Date'])
+    filWal = filWal.sort_values(by=['Date'])
+    wal = wal.sort_values(by=['Date'])
+
+    filtered_bitmex_df['Date'] = filtered_bitmex_df['Date'].dt.strftime('%m/%d/%Y')
+
+    filtered_bitmex_df['Value'] = filtered_bitmex_df['Date']
+    filtered_bitmex_df['Settlement'] = filtered_bitmex_df['Date']
+
+    filtered_bitmex_df['Depo'] = 'Bitmex'
+    filtered_bitmex_df['Bank'] = 'Bitmex'
+
+    filtered_bitmex_df['Pmt Cur'] = 'USDT'
+    filtered_bitmex_df['Source'] = 'Manual'
+
+    funding_df['Date'] = funding_df['Date'].dt.strftime('%m/%d/%Y')
+
+    funding_df['Value'] = funding_df['Date']
+    funding_df['Settlement'] = funding_df['Date']
+
+    funding_df['Depo'] = 'Bitmex'
+    funding_df['Bank'] = 'Bitmex'
+
+    funding_df['Pmt Cur'] = 'USDT'
+    funding_df['Source'] = 'Manual'
+
+    return filtered_bitmex_df,filWal,wal,actual_positions, funding_df
+
+
+def fetch_all_necessary_data_granular(public_key = None, private_key= None, start_time= None, end_time= None, me_month=None, save_to_dropbox = False, dropbox_token = None, local_root_directory ='./', recompute_all = True, by_pass_month = False):
 
     dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=dropbox_token, dropbox_backup=True)
 
